@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x65e04e94
+# __coconut_hash__ = 0xc8790ef8
 
-# Compiled with Coconut version 1.5.0-post_dev49 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
 
 """
 Utilities for use in BBopt backends.
@@ -11,17 +11,28 @@ Utilities for use in BBopt backends.
 # Coconut Header: -------------------------------------------------------------
 
 from __future__ import print_function, absolute_import, unicode_literals, division
-import sys as _coconut_sys, os.path as _coconut_os_path
-_coconut_file_path = _coconut_os_path.dirname(_coconut_os_path.dirname(_coconut_os_path.abspath(__file__)))
+import sys as _coconut_sys, os as _coconut_os
+_coconut_file_dir = _coconut_os.path.dirname(_coconut_os.path.dirname(_coconut_os.path.abspath(__file__)))
 _coconut_cached_module = _coconut_sys.modules.get(str("__coconut__"))
-if _coconut_cached_module is not None and _coconut_os_path.dirname(_coconut_cached_module.__file__) != _coconut_file_path:
+if _coconut_cached_module is not None and _coconut_os.path.dirname(_coconut_cached_module.__file__) != _coconut_file_dir:
     del _coconut_sys.modules[str("__coconut__")]
-_coconut_sys.path.insert(0, _coconut_file_path)
+_coconut_sys.path.insert(0, _coconut_file_dir)
+_coconut_module_name = _coconut_os.path.splitext(_coconut_os.path.basename(_coconut_file_dir))[0]
+if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha() or c.isdigit() for c in _coconut_module_name) and "__init__.py" in _coconut_os.listdir(_coconut_file_dir):
+    _coconut_full_module_name = str(_coconut_module_name + ".__coconut__")
+    import __coconut__ as _coconut__coconut__
+    _coconut__coconut__.__name__ = _coconut_full_module_name
+    for _coconut_v in vars(_coconut__coconut__).values():
+        if getattr(_coconut_v, "__module__", None) == str("__coconut__"):
+            try:
+                _coconut_v.__module__ = _coconut_full_module_name
+            except AttributeError:
+                _coconut_vtype = type(_coconut_v)
+                _coconut_vtype.__module__ = _coconut_full_module_name
+    _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
-if _coconut_sys.version_info >= (3,):
-    _coconut_sys.path.pop(0)
-
+_coconut_sys.path.pop(0)
 # Compiled Coconut: -----------------------------------------------------------
 
 
@@ -30,15 +41,67 @@ if _coconut_sys.version_info < (3, 3):
     from collections import Iterable
 else:
     from collections.abc import Iterable
+from contextlib import contextmanager
 
+from bbopt import constants
 from bbopt.util import sorted_items
+from bbopt.util import convert_match_errors
 from bbopt.params import param_processor
 from bbopt.registry import backend_registry
 from bbopt.registry import alg_registry
+from bbopt.registry import meta_registry
 
 
 # Utilities:
 
+@convert_match_errors
+@_coconut_mark_as_match
+def init_backend(*_coconut_match_args, **_coconut_match_kwargs):
+    """Create a backend object with the given data (backend can be backend name or class)."""
+    _coconut_match_check_0 = False
+    _coconut_FunctionMatchError = _coconut_get_function_match_error()
+    if (_coconut.sum((_coconut.len(_coconut_match_args) > 0, "backend" in _coconut_match_kwargs)) == 1) and (_coconut.sum((_coconut.len(_coconut_match_args) > 1, "examples" in _coconut_match_kwargs)) == 1) and (_coconut.sum((_coconut.len(_coconut_match_args) > 2, "params" in _coconut_match_kwargs)) == 1):
+        _coconut_match_temp_0 = _coconut_match_args[0] if _coconut.len(_coconut_match_args) > 0 else _coconut_match_kwargs.pop("backend")
+        _coconut_match_temp_1 = _coconut_match_args[1] if _coconut.len(_coconut_match_args) > 1 else _coconut_match_kwargs.pop("examples")
+        _coconut_match_temp_2 = _coconut_match_args[2] if _coconut.len(_coconut_match_args) > 2 else _coconut_match_kwargs.pop("params")
+        args = _coconut_match_args[3:]
+        _coconut_match_temp_3 = _coconut_match_kwargs.pop("attempt_to_update_backend") if "attempt_to_update_backend" in _coconut_match_kwargs else _coconut_sentinel
+        if _coconut_match_temp_3 is not _coconut_sentinel:
+            backend = _coconut_match_temp_0
+            examples = _coconut_match_temp_1
+            params = _coconut_match_temp_2
+            attempt_to_update_backend = _coconut_match_temp_3
+            options = _coconut_match_kwargs
+            _coconut_match_check_0 = True
+    if not _coconut_match_check_0:
+        raise _coconut_FunctionMatchError('match def init_backend(backend, examples, params, *args, attempt_to_update_backend, **options):', _coconut_match_args)
+
+    if isinstance(backend, type) and issubclass(backend, Backend):
+        backend_cls = backend
+    else:
+        backend_cls = backend_registry[backend]
+        assert issubclass(backend_cls, Backend), "invalid backend class for {_coconut_format_0}: {_coconut_format_1}".format(_coconut_format_0=(backend), _coconut_format_1=(backend_cls))
+
+    backend_examples = examples[:]
+    backend_params = params.copy()
+
+    new_backend = None
+    if isinstance(attempt_to_update_backend, backend_cls):
+        updated_backend = attempt_to_update_backend.attempt_update(backend_examples, backend_params, *args, **options)
+        if updated_backend is True:
+            new_backend = attempt_to_update_backend
+        elif isinstance(updated_backend, backend_cls):
+            new_backend = updated_backend
+        else:
+            assert updated_backend is False, "invalid {_coconut_format_0}.attempt_update return value: {_coconut_format_1}".format(_coconut_format_0=(backend_cls), _coconut_format_1=(updated_backend))
+
+    if new_backend is None:
+        assert not attempt_to_update_backend or isinstance(attempt_to_update_backend, Backend), "invalid backend to attempt update on: {_coconut_format_0}".format(_coconut_format_0=(attempt_to_update_backend))
+        new_backend = backend_cls(backend_examples, backend_params, *args, **options)
+
+    return new_backend
+
+
 def negate_objective(objective):
     """Take the negative of the given objective (converts a gain into a loss and vice versa)."""
     if isinstance(objective, Iterable):
@@ -55,23 +118,23 @@ def make_features(values, params, fallback_func=param_processor.choose_default_p
 # determine feature
         fallback = False
         _coconut_match_to_1 = values
-        _coconut_match_check_1 = False
+        _coconut_match_check_2 = False
         if _coconut.isinstance(_coconut_match_to_1, _coconut.abc.Mapping):
             _coconut_match_temp_0 = _coconut_match_to_1.get(name, _coconut_sentinel)
             if _coconut_match_temp_0 is not _coconut_sentinel:
                 feature = _coconut_match_temp_0
-                _coconut_match_check_1 = True
-        if _coconut_match_check_1:
+                _coconut_match_check_2 = True
+        if _coconut_match_check_2:
             pass
         else:
             _coconut_match_to_0 = kwargs
-            _coconut_match_check_0 = False
+            _coconut_match_check_1 = False
             if _coconut.isinstance(_coconut_match_to_0, _coconut.abc.Mapping):
                 _coconut_match_temp_0 = _coconut_match_to_0.get("placeholder_when_missing", _coconut_sentinel)
                 if _coconut_match_temp_0 is not _coconut_sentinel:
                     placeholder_value = _coconut_match_temp_0
-                    _coconut_match_check_0 = True
-            if _coconut_match_check_0:
+                    _coconut_match_check_1 = True
+            if _coconut_match_check_1:
                 feature = placeholder_value
             else:
                 fallback = True
@@ -80,13 +143,13 @@ def make_features(values, params, fallback_func=param_processor.choose_default_p
 # run converters
         if not fallback or convert_fallback:
             _coconut_match_to_2 = converters
-            _coconut_match_check_2 = False
+            _coconut_match_check_3 = False
             if _coconut.isinstance(_coconut_match_to_2, _coconut.abc.Mapping):
                 _coconut_match_temp_0 = _coconut_match_to_2.get(func, _coconut_sentinel)
                 if _coconut_match_temp_0 is not _coconut_sentinel:
                     converter_func = _coconut_match_temp_0
-                    _coconut_match_check_2 = True
-            if _coconut_match_check_2:
+                    _coconut_match_check_3 = True
+            if _coconut_match_check_3:
                 feature = converter_func(feature, *args)
 
         yield feature
@@ -187,23 +250,23 @@ def serve_values(name, func, args, kwargs, serving_values, fallback_func, backen
 
 # determine value
     _coconut_match_to_4 = serving_values
-    _coconut_match_check_4 = False
+    _coconut_match_check_5 = False
     if _coconut.isinstance(_coconut_match_to_4, _coconut.abc.Mapping):
         _coconut_match_temp_0 = _coconut_match_to_4.get(name, _coconut_sentinel)
         if _coconut_match_temp_0 is not _coconut_sentinel:
             value = _coconut_match_temp_0
-            _coconut_match_check_4 = True
-    if _coconut_match_check_4:
+            _coconut_match_check_5 = True
+    if _coconut_match_check_5:
         return value
     else:
         _coconut_match_to_3 = kwargs
-        _coconut_match_check_3 = False
+        _coconut_match_check_4 = False
         if _coconut.isinstance(_coconut_match_to_3, _coconut.abc.Mapping):
             _coconut_match_temp_0 = _coconut_match_to_3.get("guess", _coconut_sentinel)
             if _coconut_match_temp_0 is not _coconut_sentinel:
                 guess = _coconut_match_temp_0
-                _coconut_match_check_3 = True
-        if _coconut_match_check_3:
+                _coconut_match_check_4 = True
+        if _coconut_match_check_4:
             return guess
         else:
             return fallback_func(name, func, *args, **kwargs)
@@ -262,7 +325,7 @@ class Backend(_coconut.object):
 
     def init_fallback_backend(self):
         """Set fallback_backend to a new random backend instance."""
-        self.fallback_backend = backend_registry["random"]()
+        self.fallback_backend = backend_registry[constants.default_fallback_backend]()
 
     def fallback_func(self, name, func, *args, **kwargs):
         """Default fallback_func calls self.fallback_backend.param."""
@@ -274,23 +337,46 @@ class Backend(_coconut.object):
         assert self.current_values is not None, "Backend subclasses using Backend.param must set current_values"
         return serve_values(name, func, args, kwargs, serving_values=self.current_values, fallback_func=self.fallback_func, backend_name=self.backend_name, implemented_funcs=self.implemented_funcs, supported_kwargs=self.supported_kwargs)
 
+    registered_algs = None
+
     @classmethod
     def register(cls):
         """Register this backend to the backend registry."""
         assert cls.backend_name is not None, "Backend subclasses using Backend.register must set backend_name on the class"
         backend_registry.register(cls.backend_name, cls)
 
+# clear out registered_algs when register is called, since that
+#  probably indicates a subclass is trying to register new algs
+        cls.registered_algs = []
+
+    @classmethod
+    def register_alias(cls, alias):
+        """Register an alias for this backend."""
+        assert cls.backend_name is not None, "Backend subclasses using Backend.register_alias must set backend_name on the class"
+        backend_registry.register_alias(cls.backend_name, alias)
+
     @classmethod
     def register_alg(cls, alg_name, **options):
         """Register an algorithm under the given name that calls this backend with the given options."""
         assert cls.backend_name is not None, "Backend subclasses using Backend.register_alg must set backend_name on the class"
         alg_registry.register(alg_name, (cls.backend_name, options))
 
+        assert cls.registered_algs is not None, "Backend.register_alg must come after Backend.register"
+        cls.registered_algs.append(alg_name)
+
     @classmethod
-    def register_alias(cls, alias):
-        """Register an alias for this backend."""
-        assert cls.backend_name is not None, "Backend subclasses using Backend.register_alias must set backend_name on the class"
-        backend_registry.register_alias(cls.backend_name, alias)
+    def register_meta_for_all_algs(cls, alg_name, meta_alg=None):
+        """Register a meta algorithm for all the algs registered on this class."""
+        assert cls.registered_algs is not None, "register_meta_for_all_algs requires prior register_alg calls"
+        cls.register_meta(alg_name, cls.registered_algs, meta_alg)
+
+    @staticmethod
+    def register_meta(alg_name, algs, meta_alg=None):
+        """Register an algorithm that defers to run_meta."""
+        if meta_alg is None:
+            from bbopt.optimizer import BlackBoxOptimizer
+            meta_alg = BlackBoxOptimizer.DEFAULT_ALG_SENTINEL
+        meta_registry.register(alg_name, (algs, meta_alg))
 
     @staticmethod
     def register_param_func(func_name, handler, placeholder_generator, support_check_func):
