@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x9f5d37c8
+# __coconut_hash__ = 0xe76cccae
 
-# Compiled with Coconut version 1.5.0-post_dev49 [Fish License]
+# Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
 
 """
 The main BBopt interface.
@@ -11,17 +11,28 @@ The main BBopt interface.
 # Coconut Header: -------------------------------------------------------------
 
 from __future__ import print_function, absolute_import, unicode_literals, division
-import sys as _coconut_sys, os.path as _coconut_os_path
-_coconut_file_path = _coconut_os_path.dirname(_coconut_os_path.abspath(__file__))
+import sys as _coconut_sys, os as _coconut_os
+_coconut_file_dir = _coconut_os.path.dirname(_coconut_os.path.abspath(__file__))
 _coconut_cached_module = _coconut_sys.modules.get(str("__coconut__"))
-if _coconut_cached_module is not None and _coconut_os_path.dirname(_coconut_cached_module.__file__) != _coconut_file_path:
+if _coconut_cached_module is not None and _coconut_os.path.dirname(_coconut_cached_module.__file__) != _coconut_file_dir:
     del _coconut_sys.modules[str("__coconut__")]
-_coconut_sys.path.insert(0, _coconut_file_path)
+_coconut_sys.path.insert(0, _coconut_file_dir)
+_coconut_module_name = _coconut_os.path.splitext(_coconut_os.path.basename(_coconut_file_dir))[0]
+if _coconut_module_name and _coconut_module_name[0].isalpha() and all(c.isalpha() or c.isdigit() for c in _coconut_module_name) and "__init__.py" in _coconut_os.listdir(_coconut_file_dir):
+    _coconut_full_module_name = str(_coconut_module_name + ".__coconut__")
+    import __coconut__ as _coconut__coconut__
+    _coconut__coconut__.__name__ = _coconut_full_module_name
+    for _coconut_v in vars(_coconut__coconut__).values():
+        if getattr(_coconut_v, "__module__", None) == str("__coconut__"):
+            try:
+                _coconut_v.__module__ = _coconut_full_module_name
+            except AttributeError:
+                _coconut_vtype = type(_coconut_v)
+                _coconut_vtype.__module__ = _coconut_full_module_name
+    _coconut_sys.modules[_coconut_full_module_name] = _coconut__coconut__
 from __coconut__ import *
 from __coconut__ import _coconut_call_set_names, _coconut, _coconut_MatchError, _coconut_igetitem, _coconut_base_compose, _coconut_forward_compose, _coconut_back_compose, _coconut_forward_star_compose, _coconut_back_star_compose, _coconut_forward_dubstar_compose, _coconut_back_dubstar_compose, _coconut_pipe, _coconut_star_pipe, _coconut_dubstar_pipe, _coconut_back_pipe, _coconut_back_star_pipe, _coconut_back_dubstar_pipe, _coconut_none_pipe, _coconut_none_star_pipe, _coconut_none_dubstar_pipe, _coconut_bool_and, _coconut_bool_or, _coconut_none_coalesce, _coconut_minus, _coconut_map, _coconut_partial, _coconut_get_function_match_error, _coconut_base_pattern_func, _coconut_addpattern, _coconut_sentinel, _coconut_assert, _coconut_mark_as_match, _coconut_reiterable
-if _coconut_sys.version_info >= (3,):
-    _coconut_sys.path.pop(0)
-
+_coconut_sys.path.pop(0)
 # Compiled Coconut: -----------------------------------------------------------
 
 
@@ -35,6 +46,7 @@ else:
 import math
 import itertools
 import time
+from collections import defaultdict
 from pprint import pprint
 
 import numpy as np
@@ -42,9 +54,8 @@ import numpy as np
 from bbopt import constants
 from bbopt.registry import backend_registry
 from bbopt.registry import alg_registry
-from bbopt.params import param_processor
+from bbopt.registry import meta_registry
 from bbopt.util import Str
-from bbopt.util import init_backend
 from bbopt.util import norm_path
 from bbopt.util import json_serialize
 from bbopt.util import best_example
@@ -58,9 +69,13 @@ from bbopt.util import plot
 from bbopt.util import open_with_lock
 from bbopt.util import printerr
 from bbopt.util import convert_match_errors
+from bbopt.params import param_processor
+from bbopt.backends.util import init_backend
 from bbopt.backends.serving import ServingBackend
 
 
+# Utilities:
+
 def array_param(func, name, shape, kwargs):
     """Create a new array parameter for the given name and shape with entries from func."""
     if not isinstance(name, Str):
@@ -74,6 +89,8 @@ def array_param(func, name, shape, kwargs):
     return arr
 
 
+# Optimizer:
+
 class BlackBoxOptimizer(_coconut.object):
     """Main bbopt optimizer object. See https://github.com/evhub/bbopt for documentation."""
     DEFAULT_ALG_SENTINEL = object()
@@ -81,8 +98,7 @@ class BlackBoxOptimizer(_coconut.object):
     _new_params = None
     _current_example = None
 
-    _coconut_decorator_0 = _coconut.functools.partial(convert_match_errors, "BlackBoxOptimizer.__init__")
-    @_coconut_decorator_0
+    @convert_match_errors
     @_coconut_mark_as_match
     def __init__(*_coconut_match_args, **_coconut_match_kwargs):
         """Construct a new BlackBoxOptimizer. It is recommended to pass file=__file__."""
@@ -107,34 +123,29 @@ class BlackBoxOptimizer(_coconut.object):
 
         if protocol is None:
 # auto-detect protocol
-            self._protocol = "json"
+            self.protocol = "json"
             if not os.path.exists(self.data_file):
-                self._protocol = constants.default_protocol
+                self.protocol = constants.default_protocol
         else:
-            self._protocol = protocol
+            self.protocol = protocol
 
         self.reload()
 
 # Private utilities:
 
-    @property
-    def _using_json(self):
-        """Whether we are currently saving in json or pickle."""
-        return self._protocol == "json"
-
     def _loads(self, raw_contents):
         """Load data from the given raw data string."""
-        if self._using_json:
+        if self.using_json:
             return json.loads(str(raw_contents, encoding="utf-8"))
         else:
             return pickle.loads(raw_contents)
 
     def _dumps(self, unserialized_data):
         """Dump data to a raw data string."""
-        if self._using_json:
+        if self.using_json:
             return json.dumps((json_serialize)(unserialized_data)).encode(encoding="utf-8")
         else:
-            return pickle.dumps(unserialized_data, protocol=self._protocol)
+            return pickle.dumps(unserialized_data, protocol=self.protocol)
 
     @property
     def _got_reward(self):
@@ -152,6 +163,9 @@ class BlackBoxOptimizer(_coconut.object):
         self._current_example[reward_type] = denumpy_all(value)
         if not self.is_serving:
             self._save_current_data()
+# _save_current_data ensures that _old_params has already been
+#  updated with _new_params, so _new_params can safely be cleared
+        self._new_params = {}
 
     def _add_examples(self, examples):
         """Load the given examples into memory."""
@@ -203,30 +217,72 @@ class BlackBoxOptimizer(_coconut.object):
         ((df.write)((self._dumps)(self.get_data())))
         sync_file(df)
 
-    @property
-    def _metric(self):
-        """Whether using a gain or a loss."""
-        assert self._examples, "cannot determine metric from empty examples"
-        return "gain" if "gain" in self._examples[0] else "loss"
+    def _get_backend(self, backend, *args, **options):
+        backend_cls = backend_registry.get(backend, backend)
 
-    _skopt_backend_args = None
-    _skopt_backend = None
+        store_ind = None
+        update_backend = self.backend
+        for i, (stored_args, stored_options, stored_backend) in enumerate(self._backend_store[backend_cls]):
+            update_backend = stored_backend
+            if stored_args == args and stored_options == options:
+                store_ind = i
+                break
+
+        new_backend = init_backend(backend_cls, self._examples, self._old_params, *args, attempt_to_update_backend=update_backend, **options)
+
+        if store_ind is None:
+            self._backend_store[backend_cls].append((args, options, new_backend))
+        else:
+            self._backend_store[backend_cls][store_ind] = (args, options, new_backend)
+
+        return new_backend
 
     def _get_skopt_backend(self):
         """Get a scikit-optimize backend regardless of whether currently using one."""
         from bbopt.backends.skopt import SkoptBackend
+
         if isinstance(self.backend, SkoptBackend):
             return self.backend
+        else:
+            return self._get_backend(SkoptBackend)
 
-        skopt_backend_args = (self._examples, self._old_params)
-        if self._skopt_backend_args == skopt_backend_args:
-            return self._skopt_backend
+    @property
+    def _file_name(self):
+        """The base name of the given file."""
+        return os.path.splitext(os.path.basename(self._file))[0] + ("_" + self._tag if self._tag is not None else "")
 
-        self._skopt_backend_args = skopt_backend_args
-        self._skopt_backend = SkoptBackend(*skopt_backend_args)
-        return self._skopt_backend
+# External but undocumented:
 
-# External API:
+    def reload(self):
+        """Completely reload the optimizer."""
+        self._backend_store = defaultdict(list)
+        self._old_params = {}
+        self._examples = []
+        self._load_data()
+        self.run_backend(ServingBackend)
+
+    def save_data(self):
+        """Forcibly saves data."""
+        with open_with_lock(self.data_file) as df:
+            self._save_to(df)
+
+    @property
+    def metric(self):
+        """Whether using a gain or a loss."""
+        assert self._examples, "cannot determine metric from empty examples"
+        return "gain" if "gain" in self._examples[0] else "loss"
+
+    @property
+    def using_json(self):
+        """Whether we are currently saving in json or pickle."""
+        return self.protocol == "json"
+
+    @property
+    def num_examples(self):
+        """The number of examples seen so far (current example not counted until maximize/minimize call)."""
+        return len(self._examples)
+
+# Public API:
 
     def param(self, name, func, *args, **kwargs):
         """Create a black box parameter and return its value."""
@@ -258,16 +314,11 @@ class BlackBoxOptimizer(_coconut.object):
         self._current_example["values"][name] = value
         return value
 
-    def reload(self):
-        """Completely reload the optimizer."""
-        self._old_params = {}
-        self._examples = []
-        self._load_data()
-        self.run(alg=None)  # backend is set to serving by default
-
     def run_backend(self, backend, *args, **options):
         """Optimize parameters using the given backend."""
-        self.backend = init_backend(backend, self._examples, self._old_params, *args, attempt_to_update_backend=self.backend, **options)
+        if self._new_params:
+            raise ValueError("run must come before parameter definitions or after maximize/minimize")
+        self.backend = self._get_backend(backend, *args, **options)
         self._new_params = {}
         self._current_example = {"values": {}}
 
@@ -281,8 +332,21 @@ class BlackBoxOptimizer(_coconut.object):
         (use .algs to get the list of valid algorithms)."""
         if alg is self.DEFAULT_ALG_SENTINEL:
             alg = constants.default_alg
+        if alg in meta_registry:
+            algs, meta_alg = meta_registry[alg]
+            self.run_meta(algs, meta_alg)
+        else:
+            backend, options = alg_registry[alg]
+            self.run_backend(backend, **options)
+
+    def run_meta(self, algs, meta_alg=DEFAULT_ALG_SENTINEL):
+        """Dynamically choose the best algorithm from the given set of algorithms."""
+        if meta_alg is self.DEFAULT_ALG_SENTINEL:
+            meta_alg = constants.default_meta_alg
+        self.run(meta_alg)
+        alg = self.choice(constants.meta_opt_alg_var, algs)
         backend, options = alg_registry[alg]
-        self.run_backend(backend, **options)
+        self.backend = self._get_backend(backend, **options)
 
     def remember(self, info):
         """Store a dictionary of information about the current run."""
@@ -301,17 +365,12 @@ class BlackBoxOptimizer(_coconut.object):
     @property
     def is_serving(self):
         """Whether we are currently using the serving backend or not."""
-        return isinstance(self.backend, ServingBackend)
-
-    @property
-    def _file_name(self):
-        """The base name of the given file."""
-        return os.path.splitext(os.path.basename(self._file))[0] + ("_" + self._tag if self._tag is not None else "")
+        return isinstance(self.backend, ServingBackend) and not self.backend.allow_missing_data
 
     @property
     def data_file(self):
         """The path to the file we are saving data to."""
-        return os.path.join(os.path.dirname(self._file), self._file_name) + constants.data_file_ext + (".json" if self._using_json else ".pickle")
+        return os.path.join(os.path.dirname(self._file), self._file_name) + constants.data_file_ext + (".json" if self.using_json else ".pickle")
 
     def get_data(self, print_data=False):
         """Get all currently-loaded data as a dictionary containing params and examples."""
@@ -321,16 +380,6 @@ class BlackBoxOptimizer(_coconut.object):
             pprint(data_dict)
         return data_dict
 
-    @property
-    def num_examples(self):
-        """The number of examples seen so far (current example not counted until maximize/minimize call)."""
-        return len(self._examples)
-
-    def save_data(self):
-        """Forcibly saves data."""
-        with open_with_lock(self.data_file) as df:
-            self._save_to(df)
-
     def tell_examples(self, examples):
         """Adds the given examples to memory and writes the current memory to disk."""
         self._add_examples(examples)
@@ -357,9 +406,9 @@ class BlackBoxOptimizer(_coconut.object):
             raise ValueError("no existing data available to be plotted")
 
         iterations = range(1, len(self._examples) + 1)
-        best_metrics = ((list)((map)(_coconut.operator.itemgetter((self._metric)), (running_best)((sorted_examples)(self._examples)))))
+        best_metrics = ((list)((map)(_coconut.operator.itemgetter((self.metric)), (running_best)((sorted_examples)(self._examples)))))
 
-        return plot(iterations, best_metrics, ax=ax, yscale=yscale, title="Convergence plot for {_coconut_format_0}".format(_coconut_format_0=(self._file_name)), xlabel="Number of trials $n$", ylabel="Best {_coconut_format_0} after $n$ trials".format(_coconut_format_0=(self._metric)))
+        return plot(iterations, best_metrics, ax=ax, yscale=yscale, title="Convergence plot for {_coconut_format_0}".format(_coconut_format_0=(self._file_name)), xlabel="Number of trials $n$", ylabel="Best {_coconut_format_0} after $n$ trials".format(_coconut_format_0=(self.metric)))
 
     def plot_history(self, ax=None, yscale=None):
         """Plot the gain/loss of every point in the order in which they were sampled."""
@@ -367,13 +416,13 @@ class BlackBoxOptimizer(_coconut.object):
             raise ValueError("no existing data available to be plotted")
 
         iterations = range(1, len(self._examples) + 1)
-        metrics = ((list)((map)(_coconut.operator.itemgetter((self._metric)), (sorted_examples)(self._examples))))
+        metrics = ((list)((map)(_coconut.operator.itemgetter((self.metric)), (sorted_examples)(self._examples))))
 
-        return plot(iterations, metrics, ax=ax, yscale=yscale, title="History plot for {_coconut_format_0}".format(_coconut_format_0=(self._file_name)), xlabel="Number of trials $n$", ylabel="The {_coconut_format_0} on the $n$th trial".format(_coconut_format_0=(self._metric)))
+        return plot(iterations, metrics, ax=ax, yscale=yscale, title="History plot for {_coconut_format_0}".format(_coconut_format_0=(self._file_name)), xlabel="Number of trials $n$", ylabel="The {_coconut_format_0} on the $n$th trial".format(_coconut_format_0=(self.metric)))
 
     def partial_dependence(self, i_name, j_name=None, *args, **kwargs):
         """Calls skopt.plots.partial_dependence where i_name and j_name are parameter names."""
-        def _coconut_mock_6(self, i_name, j_name=None, *args, **kwargs): return self, i_name, j_name, args, kwargs
+        def _coconut_mock_8(self, i_name, j_name=None, *args, **kwargs): return self, i_name, j_name, args, kwargs
         while True:
             from skopt.plots import partial_dependence
             if not self._examples:
@@ -386,49 +435,85 @@ class BlackBoxOptimizer(_coconut.object):
             j = None if j_name is None else sorted_names.index(j_name)
 
             try:
-                _coconut_tre_check_0 = partial_dependence is _coconut_recursive_func_24
+                _coconut_tre_check_0 = partial_dependence is _coconut_recursive_func_26
             except _coconut.NameError:
                 _coconut_tre_check_0 = False
             if _coconut_tre_check_0:
-                self, i_name, j_name, args, kwargs = _coconut_mock_6(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
+                self, i_name, j_name, args, kwargs = _coconut_mock_8(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
                 continue
             else:
                 return partial_dependence(skopt_backend.space, skopt_backend.model, i, j, *args, **kwargs)
 
 
             return None
-    _coconut_recursive_func_24 = partial_dependence
+    _coconut_recursive_func_26 = partial_dependence
     def plot_partial_dependence_1D(self, i_name, ax=None, yscale=None, **kwargs):
         """Constructs a 1D partial dependence plot using self.partial_dependence."""
         xi, yi = self.partial_dependence(i_name, **kwargs)
         return plot(xi, yi, ax=ax, yscale=yscale, title="Partial dependence of {_coconut_format_0}".format(_coconut_format_0=(i_name)), xlabel="Values of {_coconut_format_0}".format(_coconut_format_0=(i_name)), ylabel="The loss at each point".format())
 
-    def _call_skopt_plot_func(self, skopt_plot_func, *args, **kwargs):
-        """Call the given skopt.plots function."""
+    def get_skopt_result(self):
+        """Get a result object usable by skopt.plots functions."""
         if not self._examples:
             raise ValueError("no existing data available to be plotted")
-
-        skopt_backend = self._get_skopt_backend()
-
-        return skopt_plot_func(skopt_backend.result, *args, **kwargs)
+        return self._get_skopt_backend().result
 
     def plot_evaluations(self, *args, **kwargs):
         """Calls skopt.plots.plot_evaluations."""
-        from skopt.plots import plot_evaluations
-        return self._call_skopt_plot_func(plot_evaluations, *args, **kwargs)
+        def _coconut_mock_10(self, *args, **kwargs): return self, args, kwargs
+        while True:
+            from skopt.plots import plot_evaluations
+            try:
+                _coconut_tre_check_1 = plot_evaluations is _coconut_recursive_func_29
+            except _coconut.NameError:
+                _coconut_tre_check_1 = False
+            if _coconut_tre_check_1:
+                self, args, kwargs = _coconut_mock_10(self.get_skopt_result(), *args, **kwargs)
+                continue
+            else:
+                return plot_evaluations(self.get_skopt_result(), *args, **kwargs)
 
+
+            return None
+    _coconut_recursive_func_29 = plot_evaluations
     def plot_objective(self, *args, **kwargs):
         """Calls skopt.plots.plot_objective."""
-        from skopt.plots import plot_objective
-        return self._call_skopt_plot_func(plot_objective, *args, **kwargs)
+        def _coconut_mock_11(self, *args, **kwargs): return self, args, kwargs
+        while True:
+            from skopt.plots import plot_objective
+            try:
+                _coconut_tre_check_2 = plot_objective is _coconut_recursive_func_30
+            except _coconut.NameError:
+                _coconut_tre_check_2 = False
+            if _coconut_tre_check_2:
+                self, args, kwargs = _coconut_mock_11(self.get_skopt_result(), *args, **kwargs)
+                continue
+            else:
+                return plot_objective(self.get_skopt_result(), *args, **kwargs)
 
+
+            return None
+    _coconut_recursive_func_30 = plot_objective
     def plot_regret(self, *args, **kwargs):
         """Calls skopt.plots.plot_regret."""
-        from skopt.plots import plot_regret
-        return self._call_skopt_plot_func(plot_regret, *args, **kwargs)
+        def _coconut_mock_12(self, *args, **kwargs): return self, args, kwargs
+        while True:
+            from skopt.plots import plot_regret
+            try:
+                _coconut_tre_check_3 = plot_regret is _coconut_recursive_func_31
+            except _coconut.NameError:
+                _coconut_tre_check_3 = False
+            if _coconut_tre_check_3:
+                self, args, kwargs = _coconut_mock_12(self.get_skopt_result(), *args, **kwargs)
+                continue
+            else:
+                return plot_regret(self.get_skopt_result(), *args, **kwargs)
+
 
 # Base random functions:
 
+            return None
+    _coconut_recursive_func_31 = plot_regret
     def randrange(self, name, *args, **kwargs):
         """Create a new parameter with the given name modeled by random.randrange(*args)."""
         return self.param(name, "randrange", *args, **kwargs)
@@ -478,6 +563,7 @@ class BlackBoxOptimizer(_coconut.object):
     def choice(self, name, seq, **kwargs):
         """Create a new parameter with the given name modeled by random.choice(seq)."""
         if constants.use_generic_categories_for_categorical_data:
+            (param_processor.modify_kwargs)(seq.index, kwargs)
             return seq[self._categorical(name, len(seq), **kwargs)]
         else:
             return self.param(name, "choice", seq, **kwargs)
@@ -539,6 +625,14 @@ class BlackBoxOptimizer(_coconut.object):
         random.shuffle(population) except returned instead of modified in place."""
         return self.sample(name, population, len(population), **kwargs)
 
+    def shuffle(self, name, population, **kwargs):
+        """Create a new parameter with the given name modeled by random.shuffle(population)."""
+        population[:] = self.shuffled(name, population, **kwargs)
+
+    def stdnormal(self, name, **kwargs):
+        """Equivalent to bb.normalvariate(name, 0, 1)."""
+        return self.normalvariate(name, 0, 1, **kwargs)
+
 # Array-based random functions:
 
     def rand(self, name, *shape, **kwargs):
@@ -547,6 +641,6 @@ class BlackBoxOptimizer(_coconut.object):
 
     def randn(self, name, *shape, **kwargs):
         """Create a new array parameter for the given name and shape modeled by np.random.randn."""
-        return array_param(_coconut_partial(self.normalvariate, {1: 0, 2: 1}, 3), name, shape, kwargs)
+        return array_param(self.stdnormal, name, shape, kwargs)
 
 _coconut_call_set_names(BlackBoxOptimizer)
